// For Arduino 1.0 IDE or later

const int backPack = 1;        // Pin D1 for Backpack serial
//Define constants for Backpack
#define APNDSP     0x18
#define CHGCLR     0x12
#define CLS        0x00
#define DEFWIN     0x10
#define HOME       0x01
#define LF         0x0a
#define MOVWIN     0x19
#define SHODSP     0x07
#define SHOWIN     0x1a
#define USEWIN     0x11
#define WDWRAP     0x15

#define DBL        0x80
#define SHO        0x40
#define TPT        0x01

void setup() {
  // Reset Backpack
  pinMode(backPack, OUTPUT);
  digitalWrite(backPack, LOW);
  delay(100);
  pinMode(backPack, INPUT);
  delay(2000);
  // End reset Backpack

  Serial.begin(9600);
  delay(1000);

  byte windowA = 1;
  byte displayNum = 2;
  defineWindow(windowA, 44, 2);    // Define window 44 x 2
  changeColor();
  appendDisplay(windowA, displayNum, 1, 1);
  Serial.println("This text displayed on the Backpack");
  delay(2000);                    // Wait 3 seconds
  showDisplay(0);                 // Hide text 
}

// Empty loop; for this demo everything is done in setup
void loop() { }

// Backpack helper functions follow

// Define a window, given a window number, 
//   number of columns, number of rows
void defineWindow(byte windowNum, byte cols, byte rows) {
  Serial.write(DEFWIN);
  Serial.write(windowNum);
  Serial.write(cols);
  Serial.write(rows);
}

// "Append" a window to a display, given a window number
//   display number, and x and y coordinates on the screen

void appendDisplay(byte windowNum, 
byte displayNum, byte x, byte y) {
  Serial.write(APNDSP);           // Append display to window
  Serial.write(displayNum);       // display
  Serial.write(SHO | windowNum);  // winno
  Serial.write(x);                // x
  Serial.write(y);                // y
  Serial.write(SHODSP);           // Show display
  Serial.write(displayNum);
  Serial.write(WDWRAP);           // Turn on word wrap
  Serial.write(1);
}

// Change color of text and background
void changeColor() {
  Serial.write(CHGCLR);
  Serial.write(0x0f);            // mask
  Serial.write(0x08);            // trans
  Serial.write(0x08);            // foreground
  Serial.write(0x00);            // background
}

// Move the window to a new coordinate on the screen
void moveWindow(byte displayNum, byte x, byte y) {
  Serial.write(MOVWIN);
  Serial.write(displayNum);
  Serial.write(x);
  Serial.write(y);
}

// Use (activate) a window
void useWindow(byte windowNum) {
  Serial.write(USEWIN);
  Serial.write(windowNum);
}

// Show (or hide) a display (shows or hides the text)
void showDisplay(byte displayNum) {
  Serial.write(SHODSP);
  Serial.write(displayNum);
}

// Clears all text
void cls() {
  Serial.write(CLS);
}
